<?php
/**
 * Visforms field file class
 *
 * @author       Aicha Vack
 * @package      Joomla.Site
 * @subpackage   com_visforms
 * @link         https://www.vi-solutions.de
 * @license      GNU General Public License version 2 or later; see license.txt
 * @copyright    2012 vi-solutions
 * @since        Joomla 1.6
 */

// no direct access
defined('_JEXEC') or die('Restricted access');
use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Visolutions\Component\Visforms\Site\Event\Visforms\VisfieldBeforeCreateEvent;

class VisformsFieldFile extends VisformsField
{
	public function __construct($field, $form) {
		parent::__construct($field, $form);
		// file inputs are not sumitted with the post
		$this->postValue = "";
		// no edit or queryValue for file upload fields
	}

	protected function setField() {
		// preprocessing field
		$this->extractDefaultValueParams();
		$this->extractGridSizesParams();
		$this->extractRestrictions();
		$this->mendBooleanAttribs();
        $this->setPropertiesFromFormConfiguration();
        $this->unsetDeselectedEditViewValidations();
        $this->addValidationAttribsForUpload();
        $this->setCustomPhpErrorMsg();
        PluginHelper::importPlugin('visforms');
        $dispatcher = Factory::getApplication()->getDispatcher();
        $event = new VisfieldBeforeCreateEvent('onVisfieldBeforeCreate', [
            'subject' => $this->field,
            'context' => 'com_visforms.field',
            'form' => $this->form
        ]);
        $dispatcher->dispatch('onVisfieldBeforeCreate', $event);
		$this->setIsConditional();
		$this->setFieldDefaultValue();
		$this->setCustomJs();
		$this->setFieldsetCounter();
		$this->setShowRequiredAsterix();
        $this->replaceLabelWithCustomText();
	}

	protected function setFieldDefaultValue() {

		// file upload fields do not use the post value but the $_FILES var, but if we have a POST Value, we set dataSource property
		if ((count($_POST) > 0) && isset($_POST['postid']) && ($_POST['postid'] == $this->form->id)) {
			$this->field->dataSource = 'post';
			return;
		}
		// Nothing to do
		return;
	}

	protected function setDbValue() {
		return;
	}

	protected function setRedirectParam() {
		return;
	}

	protected function setPropertiesFromFormConfiguration() {
        // if no specific maxfilesize value is set in field configuration, use value set in the form configuration
        // $maxfilesize is a number in KiloByte (byte such a big number and more difficult for users to use)
        // this value is used in the javascript validation error message
        if (!isset($this->field->maxfilesize) || (int) $this->field->maxfilesize === 0) {
            $this->field->maxfilesize = $this->form->maxfilesize;
        }
        // if no specific maxfilesize value is set in field configuration, use value set in the form configuration
        $this->field->allowedextensions = (!empty($this->field->allowedextensions)) ? $this->field->allowedextensions : $this->form->allowedextensions;
    }

	protected function addValidationAttribsForUpload() {
        // Get a valid maximum upload file size
        // if the user sets a value in the form/field configuration that is bigger than the php ini value, then the value is set to php ini value
        $maxFileSizeInByte = VisformsmediaHelper::getMaxFileSizeInByte($this->field->maxfilesize);
        // in edit view with validation disabled use php ini value
        if ($this->isEditOrSaveEditTask && (!empty($this->field->maxfilesize_nevv))) {
            $maxFileSizeInByte = VisformsmediaHelper::toBytes(ini_get('upload_max_filesize'));
        }
        // validation works with the size in byte
        // $maxFileSizeInByte is the number in bytes
		$this->field->validate_filesize = $maxFileSizeInByte;
        // add file extension validation
        if (!$this->isEditOrSaveEditTask || (empty($this->field->allowedextensions_nevv))) {
            $this->field->validate_fileextension = "'" . $this->field->allowedextensions . "'";
        }
	}
}